@@ -3,6 +3,7 @@
 // corbaFixed.cc              Created on: 07/02/2001
 //                            Author    : Duncan Grisby (dpg1)
 //
+//    Copyright (C) 2005 Apasphere Ltd
 //    Copyright (C) 2001 AT&T Laboratories Cambridge
 //
 //    This file is part of the omniORB library
@@ -28,17 +29,17 @@
 //    Implementation of the fixed point type
 
 // $Log: corbaFixed.cc,v $
-// Revision 1.1.2.16  2004/10/17 20:14:32  dgrisby
-// Updated support for OpenVMS. Many thanks to Bruce Visscher.
+// Revision 1.1.4.4  2009/05/06 16:15:30  dgrisby
+// Update lots of copyright notices.
 //
-// Revision 1.1.2.15  2004/07/23 13:22:53  dgrisby
-// Fixed point rounding was still broken.
+// Revision 1.1.4.3  2005/03/30 23:36:10  dgrisby
+// Another merge from omni4_0_develop.
 //
-// Revision 1.1.2.14  2004/07/01 19:15:17  dgrisby
-// Fix fixed point. It was broken point. Thanks Simone Viani.
+// Revision 1.1.4.2  2005/01/06 23:10:12  dgrisby
+// Big merge from omni4_0_develop.
 //
-// Revision 1.1.2.13  2003/06/02 13:26:14  dgrisby
-// Silly bug in fixed division when result has leading zeros.
+// Revision 1.1.4.1  2003/03/23 21:02:23  dgrisby
+// Start of omniORB 4.1.x development branch.
 //
 // Revision 1.1.2.12  2003/02/25 12:35:20  dgrisby
 // Typo in assertion.
@@ -242,7 +243,7 @@ CORBA::Fixed::Fixed(const CORBA::Octet* val,
 
   if (pd_digits == 0) pd_negative = 0;
 
-  memcpy(pd_val, val, digits);
+  memcpy(pd_val, val, pd_digits);
   memset(pd_val + pd_digits, 0, OMNI_FIXED_DIGITS - pd_digits);
 }
 
@@ -868,6 +869,7 @@ realMul(const CORBA::Fixed& a, const CORBA::Fixed& b, CORBA::Boolean negative)
 
     for (bi=0; bi < b.fixed_digits(); ++bi) {
       bd = b.PR_val()[bi];
+      if (bd == 0 && carry == 0) continue;
       wi       = ai + bi;
       v        = work[wi] + ad * bd + carry;
       carry    = v / 10;
