@@ -3,6 +3,7 @@
 // giopImpl10.cc              Created on: 14/02/2001
 //                            Author    : Sai Lai Lo (sll)
 //
+//    Copyright (C) 2002-2008 Apasphere Ltd
 //    Copyright (C) 2001 AT&T Laboratories, Cambridge
 //
 //    This file is part of the omniORB library
@@ -29,12 +30,36 @@
 
 /*
   $Log: giopImpl10.cc,v $
-  Revision 1.1.4.21  2004/07/01 19:16:25  dgrisby
-  Client call interceptor oneway and response expected flipped. Thanks
-  John Fardo.
+  Revision 1.1.6.10  2009/05/06 16:15:17  dgrisby
+  Update lots of copyright notices.
 
-  Revision 1.1.4.20  2003/07/25 16:07:18  dgrisby
-  Incorrect COMM_FAILURE with GIOP 1.2 CloseConnection.
+  Revision 1.1.6.9  2008/12/29 17:31:16  dgrisby
+  Properly handle message size being exceeded in request header.
+
+  Revision 1.1.6.8  2008/08/08 16:52:56  dgrisby
+  Option to validate untransformed UTF-8; correct data conversion minor
+  codes; better logging for MessageErrors.
+
+  Revision 1.1.6.7  2006/09/20 13:36:31  dgrisby
+  Descriptive logging for connection and GIOP errors.
+
+  Revision 1.1.6.6  2006/06/05 11:28:04  dgrisby
+  Change clientSendRequest interceptor members to a single GIOP_C.
+
+  Revision 1.1.6.5  2005/12/08 14:22:31  dgrisby
+  Better string marshalling performance; other minor optimisations.
+
+  Revision 1.1.6.4  2005/04/11 12:09:42  dgrisby
+  Another merge.
+
+  Revision 1.1.6.3  2005/01/06 23:10:15  dgrisby
+  Big merge from omni4_0_develop.
+
+  Revision 1.1.6.2  2003/07/10 21:55:56  dgrisby
+  Use re-entrant GIOP 1.0 size calc.
+
+  Revision 1.1.6.1  2003/03/23 21:02:16  dgrisby
+  Start of omniORB 4.1.x development branch.
 
   Revision 1.1.4.19  2003/01/22 11:40:12  dgrisby
   Correct serverSendException interceptor use.
@@ -122,9 +147,10 @@
 // the total size of the message before the actual marshalling.
 // The implementation can actually avoid this calculation if full message
 // can be fully buffered.
-// We however have to force pre-calculation because at the moment, the Any
-// marshalling implementation is not re-entrant. 
-#define PRE_CALCULATE_MESSAGE_SIZE
+// The version without pre-calculation can only work if all
+// marshalling code is re-entrant.
+
+//#define PRE_CALCULATE_MESSAGE_SIZE
 
 
 OMNI_NAMESPACE_BEGIN(omni)
@@ -139,7 +165,7 @@ public:
   static void inputMessageBegin(giopStream*,
 				void (*unmarshalHeader)(giopStream*));
   static void inputMessageEnd(giopStream*,CORBA::Boolean skip = 0);
-  static void sendMsgErrorMessage(giopStream*);
+  static void sendMsgErrorMessage(giopStream*, const CORBA::SystemException*);
   static void marshalRequestHeader(giopStream*);
   static void sendLocateRequest(giopStream*);
   static void unmarshalReplyHeader(giopStream*);
@@ -169,13 +195,15 @@ public:
   static CORBA::Boolean inputReplyBegin(giopStream*, 
 					void (*unmarshalHeader)(giopStream*));
 
-  static void inputTerminalProtocolError(giopStream* g);
+  static void inputTerminalProtocolError(giopStream* g,
+					 const char* file, int line,
+					 const char* message);
   // Helper function.  Call this function to indicate that a protocol
   // voilation was detected.  This function *always* raise a
   // giopStream::CommFailure exception.  Therefore the caller should not
   // expect this function to return.
 
-  static void inputRaiseCommFailure(giopStream* g);
+  static void inputRaiseCommFailure(giopStream* g, const char* message);
 
   static void outputNewMessage(giopStream*);
 
@@ -224,7 +252,8 @@ giopImpl10::inputMessageBegin(giopStream* g,
   if (hdr[4] != 1 || hdr[5] != 0) {
     // We accept a CloseConnection message with any GIOP version.
     if ((GIOP::MsgType)hdr[7] != GIOP::CloseConnection) {
-      inputTerminalProtocolError(g);
+      inputTerminalProtocolError(g, __FILE__, __LINE__,
+				 "Invalid GIOP message version");
       // never reaches here.
     }
   }
@@ -287,17 +316,20 @@ giopImpl10::inputReplyBegin(giopStream* g,
     {
       CORBA::ULong minor;
       CORBA::Boolean retry;
+      g->pd_strand->orderly_closed = 1;
       g->notifyCommFailure(0,minor,retry);
       g->pd_strand->state(giopStrand::DYING);
-      g->pd_strand->orderly_closed = 1;
       giopStream::CommFailure::_raise(minor,
 				      CORBA::COMPLETED_NO,
-				      retry,__FILE__,__LINE__);
+				      retry,__FILE__,__LINE__,
+				      "Orderly connection shutdown",
+				      g->pd_strand);
       // never reach here.
       break;
     }
   default:
-    inputTerminalProtocolError(g);
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Invalid reply message type");
     // never reaches here.
   }
 
@@ -308,13 +340,15 @@ giopImpl10::inputReplyBegin(giopStream* g,
     if (mtype == GIOP::LocateReply && 
 	unmarshalHeader != unmarshalLocateReply) {
 
-      inputTerminalProtocolError(g);
+      inputTerminalProtocolError(g, __FILE__, __LINE__,
+				 "Unexpected LocateReply");
       // never reach here
     }
     else if (mtype == GIOP::Reply &&
 	     unmarshalHeader != unmarshalReplyHeader) {
 
-      inputTerminalProtocolError(g);
+      inputTerminalProtocolError(g, __FILE__, __LINE__,
+				 "Unexpected Reply");
       // never reach here
     }
 
@@ -341,7 +375,8 @@ giopImpl10::inputReplyBegin(giopStream* g,
 
 	if (target->inputMatchedId()) {
 	  // a reply has already been received!
-	  inputTerminalProtocolError(g);
+	  inputTerminalProtocolError(g, __FILE__, __LINE__,
+				     "Duplicate Reply");
 	}
 
 	target->pd_input = source->pd_input;
@@ -403,7 +438,8 @@ giopImpl10::inputMessageEnd(giopStream* g,CORBA::Boolean disgard) {
 	disgard = 1;
       }
       else {
-	inputTerminalProtocolError(g);
+	inputTerminalProtocolError(g, __FILE__, __LINE__,
+				   "Garbage left at end of input message");
 	// never reach here.
       }
     }
@@ -432,7 +468,9 @@ giopImpl10::unmarshalReplyHeader(giopStream* g) {
   if ((GIOP::MsgType)hdr[7] != GIOP::Reply) {
     // Unexpected reply. The other end is terribly confused. Drop the
     // connection and died.
-    inputTerminalProtocolError(g);
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Incorrect message type when expecting a "
+			       "Reply");
     // Never reach here.
   }
 
@@ -473,7 +511,8 @@ giopImpl10::unmarshalReplyHeader(giopStream* g) {
   default:
     // Should never receive anything other that the above
     // Same treatment as wrong header
-    inputTerminalProtocolError(g);
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Invalid status in Reply");
     // Never reach here.
     break;
   }
@@ -495,7 +534,9 @@ giopImpl10::unmarshalLocateReply(giopStream* g) {
   if ((GIOP::MsgType)hdr[7] != GIOP::LocateReply) {
     // Unexpected reply. The other end is terribly confused. Drop the
     // connection and died.
-    inputTerminalProtocolError(g);
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Incorrect message type when expecting a "
+			       "LocateReply");
     // Never reach here.
   }
 
@@ -517,7 +558,8 @@ giopImpl10::unmarshalLocateReply(giopStream* g) {
   default:
     // Should never receive anything other that the above
     // Same treatment as wrong header
-    inputTerminalProtocolError(g);
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Invalid status in LocateReply");
     // Never reach here.
     break;
   }
@@ -536,7 +578,7 @@ giopImpl10::unmarshalWildCardRequestHeader(giopStream* g) {
   ((GIOP_S*)g)->requestType((GIOP::MsgType)hdr[7]);
   switch (((GIOP_S*)g)->requestType()) {
   case GIOP::CloseConnection:
-    inputRaiseCommFailure(g);
+    inputRaiseCommFailure(g, "Orderly connection shutdown");
     break;
   case GIOP::Request:
   case GIOP::LocateRequest:
@@ -544,9 +586,12 @@ giopImpl10::unmarshalWildCardRequestHeader(giopStream* g) {
     if (g->inputMessageSize() <= orbParameters::giopMaxMsgSize) {
       break;
     }
-    // falls through if the message has exceeded the size limit.
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "GIOP message size limit exceeded");
+    break;
   default:
-    inputTerminalProtocolError(g);
+    inputTerminalProtocolError(g, __FILE__, __LINE__,
+			       "Unknown GIOP message type");
     // Never reach here.
     break;
   }
@@ -873,16 +918,26 @@ giopImpl10::currentInputPtr(const giopStream* g) {
 
 ////////////////////////////////////////////////////////////////////////
 void
-giopImpl10::inputTerminalProtocolError(giopStream* g) {
+giopImpl10::inputTerminalProtocolError(giopStream* g,
+				       const char* file, int line,
+				       const char* message) {
 
-  sendMsgErrorMessage(g);
+  if (omniORB::trace(1)) {
+    omniORB::logger l;
+    l << "From endpoint: " << g->pd_strand->connection->peeraddress()
+      << ". Detected GIOP 1.0 protocol error in input message. "
+      << omniExHelper::strip(file) << ":" << line
+      << ". Connection is closed.\n";
+  }
 
-  inputRaiseCommFailure(g);
+  sendMsgErrorMessage(g, 0);
+
+  inputRaiseCommFailure(g, message);
 }
 
 ////////////////////////////////////////////////////////////////////////
 void
-giopImpl10::inputRaiseCommFailure(giopStream* g) {
+giopImpl10::inputRaiseCommFailure(giopStream* g, const char* message) {
 
   CORBA::ULong minor;
   CORBA::Boolean retry;
@@ -890,7 +945,7 @@ giopImpl10::inputRaiseCommFailure(giopStream* g) {
   g->pd_strand->state(giopStrand::DYING);
   giopStream::CommFailure::_raise(minor,
 				  (CORBA::CompletionStatus)g->completion(),
-				  0,__FILE__,__LINE__);
+				  0,__FILE__,__LINE__,message,g->pd_strand);
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -968,7 +1023,8 @@ giopImpl10::outputMessageEnd(giopStream* g) {
 
 ////////////////////////////////////////////////////////////////////////
 void
-giopImpl10::sendMsgErrorMessage(giopStream* g) {
+giopImpl10::sendMsgErrorMessage(giopStream* g,
+				const CORBA::SystemException* ex) {
 
   if (!g->pd_wrlocked) {
     omni_tracedmutex_lock sync(*omniTransportLock);
@@ -977,9 +1033,17 @@ giopImpl10::sendMsgErrorMessage(giopStream* g) {
 
   if (omniORB::trace(1)) {
     omniORB::logger l;
-    l << "To endpoint: " << g->pd_strand->connection->peeraddress()
-      << ". Send GIOP 1.0 MessageError because a protocol error has "
-      << "been detected. Connection is closed.\n";
+
+    l << "To endpoint: " << g->pd_strand->connection->peeraddress() << ". ";
+
+    if (ex) {
+      l << "System exception " << *ex << " while (un)marshalling. "
+	<< "Send GIOP 1.0 MessageError.\n";
+    }
+    else {
+      l << "Send GIOP 1.0 MessageError because a protocol error has "
+	<< "been detected. Connection is closed.\n";
+    }
   }
 
   if (!g->pd_currentOutputBuffer) {
@@ -1027,11 +1091,7 @@ giopImpl10::marshalRequestHeader(giopStream* g) {
   CORBA::Boolean response_expected = !calldesc.is_oneway();
   
 
-  omniInterceptors::clientSendRequest_T::info_T info(*g,
-				                     *(giop_c.ior()),
-						     calldesc.op(),
-						     !response_expected,
-						     response_expected);
+  omniInterceptors::clientSendRequest_T::info_T info(giop_c);
   omniInterceptorP::visit(info);
 
   {
@@ -1048,6 +1108,7 @@ giopImpl10::marshalRequestHeader(giopStream* g) {
     *((CORBA::ULong*)(hdr+8)) = cs.total();
 
 #if defined(PRE_CALCULATE_MESSAGE_SIZE)
+    omniORB::logs(30, "Pre-calculating GIOP 1.0 message size.");
     giop_c.calldescriptor()->marshalArguments(cs);
     CORBA::ULong msgsz = cs.total() - 12;
     *((CORBA::ULong*)(hdr + 8)) = msgsz;
@@ -1066,11 +1127,11 @@ giopImpl10::marshalRequestHeader(giopStream* g) {
 
   // object key
   giop_c.keysize() >>= s;
-  s.put_octet_array(giop_c.key(),giop_c.keysize());
+  s.put_small_octet_array(giop_c.key(),giop_c.keysize());
 
   // operation
   operator>>= ((CORBA::ULong)calldesc.op_len(),s);
-  s.put_octet_array((CORBA::Octet*) calldesc.op(), calldesc.op_len());
+  s.put_small_octet_array((CORBA::Octet*) calldesc.op(), calldesc.op_len());
 
   // principal
   omni::myPrincipalID >>= s;
@@ -1108,7 +1169,7 @@ giopImpl10::sendLocateRequest(giopStream* g) {
   
   // object key
   giop_c.keysize() >>= s;
-  s.put_octet_array(giop_c.key(),giop_c.keysize());
+  s.put_small_octet_array(giop_c.key(),giop_c.keysize());
 
   outputMessageEnd(g);
 }
@@ -1137,6 +1198,7 @@ giopImpl10::marshalReplyHeader(giopStream* g) {
     *((CORBA::ULong*)(hdr+8)) = cs.total();
 
 #if defined(PRE_CALCULATE_MESSAGE_SIZE)
+    omniORB::logs(30, "Pre-calculating GIOP 1.0 message size.");
     giop_s.calldescriptor()->marshalReturnedValues(cs);
     CORBA::ULong msgsz = cs.total() - 12;
     *((CORBA::ULong*)(hdr + 8)) = msgsz;
@@ -1172,14 +1234,17 @@ giopImpl10::sendSystemException(giopStream* g,const CORBA::SystemException& ex)
     if (g->outputMessageSize())
 #endif
       {
-	sendMsgErrorMessage(g);
+	sendMsgErrorMessage(g, &ex);
 
 	CORBA::ULong minor;
 	CORBA::Boolean retry;
 	giop_s.notifyCommFailure(0,minor,retry);
 	giopStream::CommFailure::_raise(minor,(CORBA::CompletionStatus)
 					giop_s.completion(),
-					retry,__FILE__,__LINE__);
+					retry,__FILE__,__LINE__,
+					"System Exception occurred while "
+					"marshalling reply. Sending a "
+					"MessageError", g->pd_strand);
       }
   }
 
@@ -1231,7 +1296,7 @@ giopImpl10::sendSystemException(giopStream* g,const CORBA::SystemException& ex)
 
   // system exception value
   CORBA::ULong(repoid_size) >>= s;
-  s.put_octet_array((const CORBA::Octet*) repoid, repoid_size);
+  s.put_small_octet_array((const CORBA::Octet*) repoid, repoid_size);
   ex.minor() >>= s;
   CORBA::ULong(ex.completed()) >>= s;
 
@@ -1289,7 +1354,7 @@ giopImpl10::sendUserException(giopStream* g,const CORBA::UserException& ex) {
 
   // user exception value
   CORBA::ULong(repoid_size) >>= s;
-  s.put_octet_array((const CORBA::Octet*) repoid, repoid_size);
+  s.put_small_octet_array((const CORBA::Octet*) repoid, repoid_size);
   ex._NP_marshal(s);
 
   outputMessageEnd(g);
