@@ -3,6 +3,7 @@
 // giopStream.cc              Created on: 16/01/2001
 //                            Author    : Sai Lai Lo (sll)
 //
+//    Copyright (C) 2002-2006 Apasphere Ltd
 //    Copyright (C) 2001 AT&T Laboratories Cambridge
 //
 //    This file is part of the omniORB library
@@ -28,17 +29,39 @@
 
 /*
   $Log: giopStream.cc,v $
-  Revision 1.1.4.28  2004/07/23 13:25:44  dgrisby
-  New traceExceptions option.
+  Revision 1.1.6.11  2009/05/06 16:15:09  dgrisby
+  Update lots of copyright notices.
 
-  Revision 1.1.4.27  2003/09/25 13:47:44  dgrisby
-  Log connection attempts.
+  Revision 1.1.6.10  2006/09/20 13:36:31  dgrisby
+  Descriptive logging for connection and GIOP errors.
 
-  Revision 1.1.4.26  2003/07/30 14:25:28  dgrisby
-  Increase direct send cut-off to 16k.
+  Revision 1.1.6.9  2006/08/17 16:22:46  dgrisby
+  Cast size_t to int in log message so it works on 64 bit Windows.
 
-  Revision 1.1.4.25  2003/07/16 14:22:38  dgrisby
-  Speed up oneway handling a little. More tracing for split messages.
+  Revision 1.1.6.8  2006/06/22 13:53:49  dgrisby
+  Add flags to strand.
+
+  Revision 1.1.6.7  2006/03/26 20:59:28  dgrisby
+  Merge from omni4_0_develop.
+
+  Revision 1.1.6.6  2006/01/10 13:59:37  dgrisby
+  New clientConnectTimeOutPeriod configuration parameter.
+
+  Revision 1.1.6.5  2005/11/17 17:03:26  dgrisby
+  Merge from omni4_0_develop.
+
+  Revision 1.1.6.4  2005/03/30 23:36:09  dgrisby
+  Another merge from omni4_0_develop.
+
+  Revision 1.1.6.3  2005/01/13 21:10:00  dgrisby
+  New SocketCollection implementation, using poll() where available and
+  select() otherwise. Windows specific version to follow.
+
+  Revision 1.1.6.2  2005/01/06 23:10:29  dgrisby
+  Big merge from omni4_0_develop.
+
+  Revision 1.1.6.1  2003/03/23 21:02:14  dgrisby
+  Start of omniORB 4.1.x development branch.
 
   Revision 1.1.4.24  2003/01/28 13:37:06  dgrisby
   Send GIOP dumps to logger. Thanks Matej Kenda.
@@ -122,12 +145,11 @@
 #include <giopStrand.h>
 #include <giopStreamImpl.h>
 #include <omniORB4/minorCode.h>
+#include <orbParameters.h>
 #include <stdio.h>
 
 OMNI_NAMESPACE_BEGIN(omni)
 
-static void dumpbuf(unsigned char* buf, size_t sz);
-
 ////////////////////////////////////////////////////////////////////////
 CORBA::ULong giopStream::directSendCutOff = 16384;
 CORBA::ULong giopStream::directReceiveCutOff = 1024;
@@ -239,7 +261,8 @@ giopStream::rdLock() {
 
     if (hastimeout) {
       // Timeout. 
-      errorOnReceive(0,__FILE__,__LINE__,0,1);
+      errorOnReceive(0,__FILE__,__LINE__,0,1,
+		     "Timed out waiting for read lock");
     }
   }
   pd_strand->rd_nwaiting = -pd_strand->rd_nwaiting - 1;
@@ -304,7 +327,8 @@ giopStream::wrLock() {
 
     if (hastimeout) {
       // Timeout. 
-      errorOnReceive(0,__FILE__,__LINE__,0,1);
+      errorOnReceive(0,__FILE__,__LINE__,0,1,
+		     "Timed out waiting for write lock");
     }
   }
   pd_strand->wr_nwaiting = -pd_strand->wr_nwaiting - 1;
@@ -394,7 +418,8 @@ giopStream::sleepOnRdLock() {
 
     if (hastimeout) {
       // Timeout. 
-      errorOnReceive(0,__FILE__,__LINE__,0,1);
+      errorOnReceive(0,__FILE__,__LINE__,0,1,
+		     "Timed out sleeping on read lock");
     }
   }
 }
@@ -447,7 +472,9 @@ giopStream::sleepOnRdLockAlways() {
 
   if (hastimeout) {
     // Timeout. 
-    errorOnReceive(0,__FILE__,__LINE__,0,1);
+    errorOnReceive(0,__FILE__,__LINE__,0,1,
+		   "Timed out sleeping on read lock "
+		   "(waiting for receiver thread)");
   }
 }
 
@@ -515,27 +542,43 @@ giopStream::CommFailure::_raise(CORBA::ULong minor,
 				CORBA::CompletionStatus status,
 				CORBA::Boolean retry,
 				const char* filename,
-				CORBA::ULong linenumber)
+				CORBA::ULong linenumber,
+				const char* message,
+				giopStrand* strand)
 {
   if (status != CORBA::COMPLETED_NO) retry = 0;
+
 #ifndef OMNIORB_NO_EXCEPTION_LOGGING
   if( omniORB::traceExceptions ) {
-    omniORB::logger l;
-    l << "throw giopStream::CommFailure from "
-      << omniExHelper::strip(filename) 	
-      << ":" << linenumber 
-      << "(" << (int)retry << ",";
-    const char* description = minorCode2String(TRANSIENT_LookupTable,minor);
-    if (!description)
-      description = minorCode2String(COMM_FAILURE_LookupTable,minor);
-    if (description)
-      l << omniORB::logger::exceptionStatus(status,description);
-    else
-      l << omniORB::logger::exceptionStatus(status,minor);
-    l << ")\n";
+    {
+      omniORB::logger l;
+      l << message << ": ";
+      if (strand->connection)
+	l << strand->connection->peeraddress();
+      else if (strand->address)
+	l << strand->address->address();
+      else
+	l << "[unknown endpoint]";
+      l << '\n';
+    }
+    {
+      omniORB::logger l;
+      l << "throw giopStream::CommFailure from "
+	<< omniExHelper::strip(filename) 	
+	<< ":" << linenumber 
+	<< "(" << (int)retry << ",";
+      const char* description = minorCode2String(TRANSIENT_LookupTable,minor);
+      if (!description)
+	description = minorCode2String(COMM_FAILURE_LookupTable,minor);
+      if (description)
+	l << omniORB::logger::exceptionStatus(status,description);
+      else
+	l << omniORB::logger::exceptionStatus(status,minor);
+      l << ")\n";
+    }
   }
 #endif
-  throw CommFailure(minor,status,retry,filename,linenumber);
+  throw CommFailure(minor,status,retry);
 }
 
 
@@ -713,7 +756,8 @@ giopStream::releaseInputBuffer(giopStream_Buffer* p) {
 ////////////////////////////////////////////////////////////////////////
 void
 giopStream::errorOnReceive(int rc, const char* filename, CORBA::ULong lineno,
-			   giopStream_Buffer* buf,CORBA::Boolean heldlock) {
+			   giopStream_Buffer* buf,CORBA::Boolean heldlock,
+			   const char* message) {
 
   CORBA::ULong minor;
   CORBA::Boolean retry;
@@ -734,8 +778,9 @@ giopStream::errorOnReceive(int rc, const char* filename, CORBA::ULong lineno,
     pd_strand->state(giopStrand::DYING);
   }
   if (buf) giopStream_Buffer::deleteBuffer(buf);
+
   CommFailure::_raise(minor,(CORBA::CompletionStatus)completion(),retry,
-		      filename,lineno);
+		      filename,lineno, message, pd_strand);
   // never reaches here.
 }
 
@@ -756,7 +801,8 @@ giopStream::ensureSaneHeader(const char* filename, CORBA::ULong lineno,
     notifyCommFailure(0,minor,retry);
     giopStream_Buffer::deleteBuffer(buf);
     CommFailure::_raise(minor,(CORBA::CompletionStatus)completion(),retry,
-			filename,lineno);
+			filename,lineno,
+			"Input message is not a GIOP message", pd_strand);
     // never reaches here.
   }
   // Get the message size from the buffer
@@ -796,7 +842,8 @@ giopStream::inputMessage() {
     else {
       status = (CORBA::CompletionStatus)completion();
     }
-    CommFailure::_raise(minor,status,retry,__FILE__,__LINE__);
+    CommFailure::_raise(minor,status,retry,__FILE__,__LINE__,
+			"Connection is dying", pd_strand);
     // never reaches here.
   }
 
@@ -828,7 +875,8 @@ giopStream::inputMessage() {
       buf->last += rsz;
     }
     else {
-      errorOnReceive(rsz,__FILE__,__LINE__,buf,0);
+      errorOnReceive(rsz,__FILE__,__LINE__,buf,0,
+		     "Error in network receive (start of message)");
       // never reaches here.
     }
   }
@@ -873,7 +921,8 @@ giopStream::inputMessage() {
 	total -= rsz;
       }
       else {
-	errorOnReceive(rsz,__FILE__,__LINE__,buf,0);
+	errorOnReceive(rsz,__FILE__,__LINE__,buf,0,
+		       "Error in network receive (continuation of message)");
 	// never reaches here.
       }
     }
@@ -936,9 +985,9 @@ giopStream::inputMessage() {
     } while (first != buf->last);
     buf->last = buf->start + buf->size;
 
-    if (omniORB::trace(30)) {
+    if (omniORB::trace(25)) {
       omniORB::logger log;
-      log << "Split input data into " << splitcount << " messages\n";
+      log << "Split input data into " << splitcount+1 << " messages\n";
     }
   }
   return buf;
@@ -961,7 +1010,9 @@ giopStream::inputChunk(CORBA::ULong maxsize) {
     CORBA::Boolean retry;
     notifyCommFailure(0,minor,retry);
     CommFailure::_raise(minor,(CORBA::CompletionStatus)completion(),retry,
-			__FILE__,__LINE__);
+			__FILE__,__LINE__,
+			"New message received in the middle of an existing "
+			"message", pd_strand);
     // never reaches here.
   }
   else if (pd_strand->spare) {
@@ -989,7 +1040,8 @@ giopStream::inputChunk(CORBA::ULong maxsize) {
       maxsize -= rsz;
     }
     else {
-      errorOnReceive(rsz,__FILE__,__LINE__,buf,0);
+      errorOnReceive(rsz,__FILE__,__LINE__,buf,0,
+		     "Error in network receive (input of chunk)");
       // never reaches here.
     }
   }
@@ -1020,7 +1072,9 @@ giopStream::inputCopyChunk(void* dest, CORBA::ULong size) {
     CORBA::Boolean retry;
     notifyCommFailure(0,minor,retry);
     CommFailure::_raise(minor,(CORBA::CompletionStatus)completion(),retry,
-			__FILE__,__LINE__);
+			__FILE__,__LINE__,
+			"New message received in the middle of an existing "
+			"message (bulk receive)", pd_strand);
     // never reaches here.
   }
 
@@ -1046,7 +1100,8 @@ giopStream::inputCopyChunk(void* dest, CORBA::ULong size) {
 
     }
     else {
-      errorOnReceive(rsz,__FILE__,__LINE__,0,0);
+      errorOnReceive(rsz,__FILE__,__LINE__,0,0,
+		     "Error in receive (input of bulk data)");
       // never reaches here.
     }
   }
@@ -1060,17 +1115,42 @@ giopStream::sendChunk(giopStream_Buffer* buf) {
     OMNIORB_ASSERT(pd_strand->address);
       
     if (pd_strand->state() != giopStrand::DYING) {
-      if (omniORB::trace(25)) {
+      if (omniORB::trace(20)) {
 	omniORB::logger log;
 	log << "Client attempt to connect to "
 	    << pd_strand->address->address() << "\n";
       }
-      giopActiveConnection* c = pd_strand->address->Connect(pd_deadline_secs,
-							 pd_deadline_nanosecs);
+      unsigned long deadline_secs, deadline_nanosecs;
+
+      if (orbParameters::clientConnectTimeOutPeriod.secs ||
+	  orbParameters::clientConnectTimeOutPeriod.nanosecs) {
+
+	omni_thread::
+	  get_time(&deadline_secs,
+		   &deadline_nanosecs,
+		   orbParameters::clientConnectTimeOutPeriod.secs,
+		   orbParameters::clientConnectTimeOutPeriod.nanosecs);
+
+	if ((pd_deadline_secs && deadline_secs > pd_deadline_secs) ||
+	    (deadline_secs == pd_deadline_secs &&
+	     deadline_nanosecs > pd_deadline_nanosecs)) {
+
+	  pd_deadline_secs     = deadline_secs;
+	  pd_deadline_nanosecs = deadline_nanosecs;
+	}
+      }
+      else {
+	deadline_secs     = pd_deadline_secs;
+	deadline_nanosecs = pd_deadline_nanosecs;
+      }
+      giopActiveConnection* c = pd_strand->address->Connect(deadline_secs,
+							    deadline_nanosecs,
+							    pd_strand->flags);
       if (c) pd_strand->connection = &(c->getConnection());
     }
     if (!pd_strand->connection) {
-      errorOnSend(TRANSIENT_ConnectFailed,__FILE__,__LINE__,0);
+      errorOnSend(TRANSIENT_ConnectFailed,__FILE__,__LINE__,0,
+		  "Unable to open new connection");
     }
     if (omniORB::trace(20)) {
       omniORB::logger log;
@@ -1103,7 +1183,8 @@ giopStream::sendChunk(giopStream_Buffer* buf) {
       first += ssz;
     }
     else {
-      errorOnSend(ssz,__FILE__,__LINE__,0);
+      errorOnSend(ssz,__FILE__,__LINE__,0,
+		  "Error in network send");
       // never reaches here.
     }
   }
@@ -1116,12 +1197,41 @@ giopStream::sendCopyChunk(void* buf,CORBA::ULong size) {
   if (!pd_strand->connection) {
     OMNIORB_ASSERT(pd_strand->address);
     if (pd_strand->state() != giopStrand::DYING) {
-      giopActiveConnection* c = pd_strand->address->Connect(pd_deadline_secs,
-							    pd_deadline_nanosecs);
+      if (omniORB::trace(20)) {
+	omniORB::logger log;
+	log << "Client attempt to connect to "
+	    << pd_strand->address->address() << "\n";
+      }
+      unsigned long deadline_secs, deadline_nanosecs;
+
+      if (orbParameters::clientConnectTimeOutPeriod.secs ||
+	  orbParameters::clientConnectTimeOutPeriod.nanosecs) {
+
+	omni_thread::
+	  get_time(&deadline_secs,
+		   &deadline_nanosecs,
+		   orbParameters::clientConnectTimeOutPeriod.secs,
+		   orbParameters::clientConnectTimeOutPeriod.nanosecs);
+
+	if ((pd_deadline_secs && deadline_secs > pd_deadline_secs) ||
+	    (deadline_secs == pd_deadline_secs &&
+	     deadline_nanosecs > pd_deadline_nanosecs)) {
+
+	  pd_deadline_secs     = deadline_secs;
+	  pd_deadline_nanosecs = deadline_nanosecs;
+	}
+      }
+      else {
+	deadline_secs     = pd_deadline_secs;
+	deadline_nanosecs = pd_deadline_nanosecs;
+      }
+      giopActiveConnection* c = pd_strand->address->Connect(deadline_secs,
+							    deadline_nanosecs);
       if (c) pd_strand->connection = &(c->getConnection());
     }
     if (!pd_strand->connection) {
-      errorOnSend(TRANSIENT_ConnectFailed,__FILE__,__LINE__,0);
+      errorOnSend(TRANSIENT_ConnectFailed,__FILE__,__LINE__,0,
+		  "Unable to open new connection");
     }
     if (omniORB::trace(20)) {
       omniORB::logger log;
@@ -1151,7 +1261,8 @@ giopStream::sendCopyChunk(void* buf,CORBA::ULong size) {
       buf = (void*)((omni::ptr_arith_t)buf + ssz);
     }
     else {
-      errorOnSend(ssz,__FILE__,__LINE__,0);
+      errorOnSend(ssz,__FILE__,__LINE__,0,
+		  "Error in network send (output of bulk data)");
       // never reaches here.
     }
   }
@@ -1161,7 +1272,7 @@ giopStream::sendCopyChunk(void* buf,CORBA::ULong size) {
 ////////////////////////////////////////////////////////////////////////
 void
 giopStream::errorOnSend(int rc, const char* filename, CORBA::ULong lineno,
-			CORBA::Boolean heldlock) {
+			CORBA::Boolean heldlock, const char* message) {
 
   CORBA::ULong minor;
   CORBA::Boolean retry;
@@ -1177,12 +1288,25 @@ giopStream::errorOnSend(int rc, const char* filename, CORBA::ULong lineno,
   else if (rc == TRANSIENT_ConnectFailed) {
     pd_strand->state(giopStrand::DYING);
     minor = rc;
+
+    if (pd_deadline_secs || pd_deadline_nanosecs) {
+      // Did we timeout?
+      unsigned long s, ns;
+      omni_thread::get_time(&s, &ns);
+      if (s > pd_deadline_secs ||
+          (s == pd_deadline_secs && ns > pd_deadline_nanosecs)) {
+        
+        retry = 0;
+        minor = TRANSIENT_CallTimedout;
+      }
+    }
   }
   else {
     pd_strand->state(giopStrand::DYING);
   }
+
   CommFailure::_raise(minor,(CORBA::CompletionStatus)completion(),retry,
-		      filename,lineno);
+		      filename,lineno,message,pd_strand);
   // never reaches here.
 }
 
@@ -1193,7 +1317,8 @@ static inline char printable_char(char c) {
 }
 
 /////////////////////////////////////////////////////////////////////////
-static void dumpbuf(unsigned char* buf, size_t sz)
+void 
+giopStream::dumpbuf(unsigned char* buf, size_t sz)
 {
   static omni_tracedmutex lock;
   omni_tracedmutex_lock sync(lock);
@@ -1203,7 +1328,7 @@ static void dumpbuf(unsigned char* buf, size_t sz)
 
   const size_t dumplimit = 128;
   if (!omniORB::trace(40) && sz > dumplimit) {
-    l << dumplimit << "bytes out of " << sz << "\n";
+    l << (int)dumplimit << " bytes out of " << (int)sz << "\n";
     sz = dumplimit;
   }
   else {
